-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftNameText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Name Text"] = true,
	["Nameplate text options"] = true,
	["Enable"] = true,
	["Show unit names"] = true,
	
	["Typeface"] = true,
	["Nameplate text typeface options"] = true,
	["Font"] = true,
	["Sets the font for nameplate text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on nameplate text"] = true,
	["Font Size"] = true,
	["Sets the font height of the nameplate text"] = true,
	["Outline"] = true,
	["Sets the font outline for nameplate text"] = true,

	["Alignment"] = true,
	["Sets the alignment of the nameplate text"] = true,	

	["Position"] = true,
	["Adjust nameplate text position"] = true,
	["Left Offset"] = true,
	["Sets the offset of the left of the text"] = true,
	["Right Offset"] = true,
	["Sets the offset of the right of the text"] = true,
	["Vertical Offset"] = true,
	["Sets the vertical offset of the text"] = true,
	
	["Override Colors"] = true,
	["Override the games colors for nameplate text"] = true,
	["Colors"] = true,
	["Customize colors"] = true,

	["Color by Class"] = true,
	["Sets the name text color according to class"] = true,
	["Color Hostile by Class"] = true,
	["Sets the name text color of hostile players according to class"] = true,
	["Set All"] = true,
	["Sets the name text color for all types"] = true,
	["Set All Friendly"] = true,
	["Sets the name text color for all friendly types"] = true,
	["Set All Hostile"] = true,
	["Sets the name text color for all hostile types"] = true,

	["Pet"] = true,
	["Sets the name text color for your pet"] = true,
	["Friendly Players"] = true,
	["Sets the name text color for friendly players"] = true,
	["Friendly Pets"] = true,
	["Sets the name text color for friendly pets"] = true,
	["Friendly NPCs"] = true,
	["Sets the name text color for friendly NPC Bosses"] = true,
	["Friendly Boss"] = true,
	["Sets the name text color for friendly NPCs"] = true,
	["Hostile Players"] = true,
	["Sets the name text color for hostile players"] = true,
	["Hostile Pets"] = true,
	["Sets the name text color for hostile pets"] = true,
	["Hostile NPCs"] = true,
	["Sets the name text color for hostile units"] = true,
	["Hostile Boss"] = true,
	["Sets the name text color for hostile boss units"] = true,
	["Neutral Units"] = true,
	["Sets the name text color for neutral units"] = true,
	["Group Pets"] = true,
	["Sets the name text color for group pets"] = true,

	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,

	["Advanced"] = true,
	["Options for expert users"] = true,
	["Format"] = true,
	["Name tag"] = true,
	["<Any tag string>"] = true,
} end)

-----------------------------------------------------------------------------

local AloftNameText = Aloft:NewModule("NameText", "AceEvent-2.0")
AloftNameText.db = Aloft:AcquireDBNamespace("nameText")
Aloft:RegisterDefaults("nameText", "profile", {
	enable			= true,
	font			= "Arial Narrow",
	fontSize		= 10,
	format			= "[name]",
	shadow			= false,
	alignment		= "LEFT",
	outline			= "OUTLINE",
	offsets =
	{
		left		= 0,
		right		= 0,
		vertical	= 0,
	},
	overrideColors		= true,
	colorByClass		= false,
	colorHostileByClass	= false,
	colors =
	{
		friendlyPlayer	= { 1, 1, 1 },
		friendlyPet		= { 1, 1, 1 },
		friendlyNPC		= { 1, 1, 1 },
		friendlyBoss	= { 1, 1, 1 },
		hostilePlayer	= { 1, 1, 1 },
		hostilePet		= { 1, 1, 1 },
		hostileNPC		= { 1, 1, 1 },
		hostileBoss		= { 1, 1, 1 },
		neutral			= { 1, 1, 1 },
		pet				= { 1, 1, 1 },
		groupPet		= { 1, 1, 1 },
	},
})

Aloft.Options.args.nameText =
{
	type = 'group',
	name = L["Name Text"],
	desc = L["Nameplate text options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show unit names"],
			get = function() return AloftNameText.db.profile.enable end,
			set = function(v) AloftNameText.db.profile.enable = v Aloft:DetermineDataSources() AloftNameText:UpdateAll() end,
			order = 1
		},
		typeface =
		{
			type = "group",
			name = L["Typeface"],
			desc = L["Nameplate text typeface options"],
			disabled = function() return not AloftNameText.db.profile.enable end,
			args =
			{
				font =
				{
					type = "text", 
					name = L["Font"], 
					desc = L["Sets the font for nameplate text"],
					get = function() return AloftNameText.db.profile.font end,
					set = function(v) AloftNameText.db.profile.font = v AloftNameText:UpdateAll() end,
					validate = SML_font,
				},
				fontSize =
				{
					type = 'range',
					name = L["Font Size"],
					desc = L["Sets the font height of the nameplate text"],
					max = 16,
					min = 4,
					step = 1,
					get = function() return AloftNameText.db.profile.fontSize end,
					set = function(value) AloftNameText.db.profile.fontSize = value AloftNameText:UpdateAll() end
				},
				shadow =
				{
					type = 'toggle',
					name = L["Font Shadow"],
					desc = L["Show font shadow on nameplate text"],
					get = function() return AloftNameText.db.profile.shadow end,
					set = function(v) AloftNameText.db.profile.shadow = v AloftNameText:UpdateAll() end
				},
				outline =
				{
					type = 'text',
					name = L["Outline"],
					desc = L["Sets the font outline for nameplate text"],
					get = function() return AloftNameText.db.profile.outline end,
					set = function(value) AloftNameText.db.profile.outline = value AloftNameText:UpdateAll() end,
					validate = { [""] = L["None"], ["OUTLINE"] = L["Normal"], ["THICKOUTLINE"] = L["Thick"] },
				},
			},
		},
		alignment =
		{
			type = 'text',
			name = L["Alignment"],
			desc = L["Sets the alignment of the nameplate text"],
			disabled = function() return not AloftNameText.db.profile.enable end,
			get = function() return AloftNameText.db.profile.alignment end,
			set = function(value) AloftNameText.db.profile.alignment = value AloftNameText:UpdateAll() end,
			validate = {"LEFT", "CENTER", "RIGHT" },
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust nameplate text position"],
			disabled = function() return not AloftNameText.db.profile.enable end,
			args =
			{
				left =
				{
					type = 'range',
					name = L["Left Offset"],
					desc = L["Sets the offset of the left of the text"],
					min = -100,
					max = 100,
					step = 1,
					get = function() return AloftNameText.db.profile.offsets.left end,
					set = function(v) AloftNameText.db.profile.offsets.left = v AloftNameText:UpdateAll() end
				},
				right =
				{
					type = 'range',
					name = L["Right Offset"],
					desc = L["Sets the offset of the right of the text"],
					min = -100,
					max = 100,
					step = 1,
					get = function() return AloftNameText.db.profile.offsets.right end,
					set = function(v) AloftNameText.db.profile.offsets.right = v AloftNameText:UpdateAll() end
				},
				vertical =
				{
					type = 'range',
					name = L["Vertical Offset"],
					desc = L["Sets the vertical offset of the text"],
					min = -64,
					max = 64,
					step = 1,
					get = function() return AloftNameText.db.profile.offsets.vertical end,
					set = function(v) AloftNameText.db.profile.offsets.vertical = v AloftNameText:UpdateAll() end
				},
			},
		},
		overrideColors =
		{
			type = 'toggle',
			name = L["Override Colors"],
			desc = L["Override the games colors for nameplate text"],
			disabled = function() return not AloftNameText.db.profile.enable end,
			get = function() return AloftNameText.db.profile.overrideColors end,
			set = function(value) AloftNameText.db.profile.overrideColors = value AloftNameText:UpdateAll() end,
		},
		colors =
		{
			type = 'group',
			name = L["Colors"],
			desc = L["Customize colors"],
			disabled = function() return not AloftNameText.db.profile.enable or not AloftNameText.db.profile.overrideColors end,
			args =
			{
				colorByClass =
				{
					type = 'toggle',
					name = L["Color by Class"],
					desc = L["Sets the name text color according to class"],
					get = function() return AloftNameText.db.profile.colorByClass end,
					set = function(v) AloftNameText.db.profile.colorByClass = v Aloft:DetermineDataSources() AloftNameText:UpdateAll() end,
					order = 1
				},
				colorHostileByClass =
				{
					type = 'toggle',
					name = L["Color Hostile by Class"],
					desc = L["Sets the name text color of hostile players according to class"],
					get = function() return AloftNameText.db.profile.colorHostileByClass end,
					set = function(v) AloftNameText.db.profile.colorHostileByClass = v Aloft:DetermineDataSources() AloftNameText:UpdateAll() end,
					order = 2
				},
				setAll =
				{
					type = 'color',
					name = L["Set All"],
					desc = L["Sets the name text color for all types"],
					get = function() return unpack(AloftNameText.db.profile.colors.friendlyPlayer) end,
					set = function(r, g, b) 
						AloftNameText.db.profile.colors.friendlyPlayer	= { r, g, b }
						AloftNameText.db.profile.colors.friendlyPet		= { r, g, b }
						AloftNameText.db.profile.colors.friendlyNPC		= { r, g, b }
						AloftNameText.db.profile.colors.friendlyBoss	= { r, g, b }
						AloftNameText.db.profile.colors.hostilePlayer	= { r, g, b }
						AloftNameText.db.profile.colors.hostilePet		= { r, g, b }
						AloftNameText.db.profile.colors.hostileNPC		= { r, g, b }
						AloftNameText.db.profile.colors.hostileBoss		= { r, g, b }
						AloftNameText.db.profile.colors.neutral			= { r, g, b }
						AloftNameText.db.profile.colors.pet				= { r, g, b }
						AloftNameText.db.profile.colors.groupPet		= { r, g, b }
						Aloft:DetermineDataSources() 
						AloftNameText:UpdateAll() end,
					order = 3
				},
				setAllFriendly =
				{
					type = 'color',
					name = L["Set All Friendly"],
					desc = L["Sets the name text color for all friendly types"],
					get = function() return unpack(AloftNameText.db.profile.colors.friendlyPlayer) end,
					set = function(r, g, b) 
						AloftNameText.db.profile.colors.friendlyPlayer	= { r, g, b }
						AloftNameText.db.profile.colors.friendlyPet		= { r, g, b }
						AloftNameText.db.profile.colors.friendlyNPC		= { r, g, b }
						AloftNameText.db.profile.colors.friendlyBoss	= { r, g, b }
						AloftNameText.db.profile.colors.pet				= { r, g, b }
						AloftNameText.db.profile.colors.groupPet		= { r, g, b }
						Aloft:DetermineDataSources() 
						AloftNameText:UpdateAll() end,
					order = 4
				},
				setAllHostile =
				{
					type = 'color',
					name = L["Set All Hostile"],
					desc = L["Sets the name text color for all hostile types"],
					get = function() return unpack(AloftNameText.db.profile.colors.hostileNPC) end,
					set = function(r, g, b) 
						AloftNameText.db.profile.colors.hostilePlayer	= { r, g, b }
						AloftNameText.db.profile.colors.hostilePet		= { r, g, b }
						AloftNameText.db.profile.colors.hostileNPC		= { r, g, b }
						AloftNameText.db.profile.colors.hostileBoss		= { r, g, b }
						Aloft:DetermineDataSources() 
						AloftNameText:UpdateAll() end,
					order = 5
				},
				friendlyPlayer =
				{
					type = 'color',
					name = L["Friendly Players"],
					desc = L["Sets the name text color for friendly players"],
					get = function() return unpack(AloftNameText.db.profile.colors.friendlyPlayer) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.friendlyPlayer = { r, g, b } AloftNameText:UpdateAll() end
				},
				friendlyPet =
				{
					type = 'color',
					name = L["Friendly Pets"],
					desc = L["Sets the name text color for friendly pets"],
					get = function() return unpack(AloftNameText.db.profile.colors.friendlyPet) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.friendlyPet = { r, g, b } Aloft:DetermineDataSources() AloftNameText:UpdateAll() end
				},
				friendlyNPC =
				{
					type = 'color',
					name = L["Friendly NPCs"],
					desc = L["Sets the name text color for friendly NPCs"],
					get = function() return unpack(AloftNameText.db.profile.colors.friendlyNPC) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.friendlyNPC = { r, g, b } Aloft:DetermineDataSources() AloftNameText:UpdateAll() end
				},
				friendlyBoss =
				{
					type = 'color',
					name = L["Friendly Boss"],
					desc = L["Sets the name text color for friendly NPC Bosses"],
					get = function() return unpack(AloftNameText.db.profile.colors.friendlyBoss) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.friendlyBoss = { r, g, b } AloftNameText:UpdateAll() end
				},
				hostilePlayer =
				{
					type = 'color',
					name = L["Hostile Players"],
					desc = L["Sets the name text color for hostile players"],
					disabled = function() return AloftNameText.db.profile.colorHostileByClass end,
					get = function() return unpack(AloftNameText.db.profile.colors.hostilePlayer) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.hostilePlayer = { r, g, b } Aloft:DetermineDataSources() AloftNameText:UpdateAll() end
				},
				hostilePet  =
				{
					type = 'color',
					name = L["Hostile Pets"],
					desc = L["Sets the name text color for hostile pets"],
					get = function() return unpack(AloftNameText.db.profile.colors.hostilePet) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.hostilePet = { r, g, b } Aloft:DetermineDataSources() AloftNameText:UpdateAll() end
				},
				hostileNPC =
				{
					type = 'color',
					name = L["Hostile NPCs"],
					desc = L["Sets the name text color for hostile units"],
					get = function() return unpack(AloftNameText.db.profile.colors.hostileNPC) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.hostileNPC = { r, g, b } Aloft:DetermineDataSources() AloftNameText:UpdateAll() end
				},
				hostileBoss =
				{
					type = 'color',
					name = L["Hostile Boss"],
					desc = L["Sets the name text color for hostile boss units"],
					get = function() return unpack(AloftNameText.db.profile.colors.hostileBoss) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.hostileBoss = { r, g, b } AloftNameText:UpdateAll() end
				},
				neutral =
				{
					type = 'color',
					name = L["Neutral Units"],
					desc = L["Sets the name text color for neutral units"],
					get = function() return unpack(AloftNameText.db.profile.colors.neutral) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.neutral = { r, g, b } AloftNameText:UpdateAll() end
				},
				pet =
				{
					type = 'color',
					name = L["Pet"],
					desc = L["Sets the name text color for your pet"],
					get = function() return unpack(AloftNameText.db.profile.colors.pet) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.pet = { r, g, b } AloftNameText:UpdateAll() end
				},
				groupPet =
				{
					type = 'color',
					name = L["Group Pets"],
					desc = L["Sets the name text color for group pets"],
					get = function() return unpack(AloftNameText.db.profile.colors.groupPet) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.groupPet = { r, g, b } Aloft:DetermineDataSources() AloftNameText:UpdateAll() end
				},
			}
		},
		advanced  =
		{
			type = 'group',
			name = L["Advanced"],
			desc = L["Options for expert users"],
			order = -1,
			disabled = function() return not AloftNameText.db.profile.enable end,
			args =
			{
				format = 
				{
					type = 'text',
					name = L["Format"],
					desc = L["Name tag"],
					usage = L["<Any tag string>"],
					get = function() return AloftNameText.db.profile.format end,
					set = function(v) AloftNameText.db.profile.format = v Aloft:DetermineDataSources() AloftNameText:UpdateAll() end,
				}
			},
		},
	},
}

-----------------------------------------------------------------------------

local textMethod = nil
local textMethodData = { }

-----------------------------------------------------------------------------

function AloftNameText:UpdateAll()
	for aloftData in Aloft:IterateNameplates() do
		self:SetupFrame(aloftData)
	end

	if self.db.profile.enable then
		for aloftData in Aloft:IterateVisibleNameplates() do
			if self.db.profile.overrideColors then
				self:UpdateText(aloftData)
			else
				self:UpdateTextNone(aloftData)
			end
		end
	end
end

function AloftNameText:RegisterEvents()
	local profile = self.db.profile

	self:UnregisterAllEvents()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:UpdateAll()
		end
	end)
	
	if profile.enable and profile.overrideColors then
		local updateTextMethod = "UpdateTextSimple"
		if not self:IsArrayEqual(profile.colors.friendlyPlayer, profile.colors.friendlyPet) or
		   not self:IsArrayEqual(profile.colors.friendlyPlayer, profile.colors.friendlyNPC) or
		   not self:IsArrayEqual(profile.colors.friendlyPlayer, profile.colors.friendlyBoss) or
		   not self:IsArrayEqual(profile.colors.friendlyPlayer, profile.colors.hostilePlayer) or
		   not self:IsArrayEqual(profile.colors.friendlyPlayer, profile.colors.hostilePet) or
		   not self:IsArrayEqual(profile.colors.friendlyPlayer, profile.colors.hostileNPC) or
		   not self:IsArrayEqual(profile.colors.friendlyPlayer, profile.colors.hostileBoss) or
		   not self:IsArrayEqual(profile.colors.friendlyPlayer, profile.colors.neutral) or
		   not self:IsArrayEqual(profile.colors.friendlyPlayer, profile.colors.pet) or
		   not self:IsArrayEqual(profile.colors.friendlyPlayer, profile.colors.groupPet) or
		   profile.colorByClass or
		   profile.colorHostileByClass then
			updateTextMethod = "UpdateText"
		end

		self:RegisterEvent("Aloft:OnNameplateShow", updateTextMethod)
		self:RegisterEvent("Aloft:OnHealthBarColorChanged", updateTextMethod)

		if profile.colorByClass or profile.colorHostileByClass then
			self:RegisterEvent("Aloft:OnClassDataChanged", updateTextMethod)
		end

		if not self:IsArrayEqual(profile.colors.friendlyPet, profile.colors.friendlyNPC) or
		   not self:IsArrayEqual(profile.colors.hostilePet, profile.colors.hostileNPC) then
			self:RegisterEvent("Aloft:OnIsPetDataChanged", updateTextMethod)
		end

		for events in pairs(textMethodData.events) do
			self:RegisterEvent(events, updateTextMethod)
		end
	else
		if profile.enable then
			self:RegisterEvent("Aloft:OnNameplateShow", "UpdateTextNone")

			for events in pairs(textMethodData.events) do
				self:RegisterEvent(events, "UpdateTextNone")
			end
		end
	end
end

-----------------------------------------------------------------------------

function AloftNameText:OnEnable()
	self:UpdateAll()
end

function AloftNameText:OnDisable()
	self:UnregisterAllEvents()

	for aloftData in Aloft:IterateNameplates() do
		local nameTextRegion = aloftData.nameTextRegion

		nameTextRegion:ClearAllPoints()
		nameTextRegion:SetPoint("BOTTOM", aloftData.nameplateFrame, "CENTER")
		nameTextRegion:SetFont(NAMEPLATE_FONT, 14.5)		
		nameTextRegion:SetText(aloftData.name)
		nameTextRegion:SetShadowOffset(1, -1)
		nameTextRegion:Show()
	end
end

local dataRequiredList = { }
function AloftNameText:RequiresData()
	local profile = self.db.profile

	textMethodData = AloftTag:CreateTag(profile.format)
	textMethod = textMethodData.method
	
	self:RegisterEvents()
	
	if not profile.enable then
		return
	end

	for i = 1,#dataRequiredList do
		dataRequiredList[i] = nil
	end

	if profile.colorByClass or
	   profile.colorHostileByClass or
	   not self:IsArrayEqual(profile.colors.hostileNPC, profile.colors.hostilePlayer) then
		table.insert(dataRequiredList, "class")
	end

	if not self:IsArrayEqual(profile.colors.friendlyPet, profile.colors.friendlyNPC) or
	   not self:IsArrayEqual(profile.colors.hostilePet, profile.colors.hostileNPC) then
		table.insert(dataRequiredList, "isPet")
	end

	if Aloft:IsDataAvailable("unitid") and not self:IsArrayEqual(profile.colors.friendlyPet, profile.colors.groupPet) then
		table.insert(dataRequiredList, "unitid")
	end

	for data in pairs(textMethodData.data) do
		table.insert(dataRequiredList, data)
	end
	
	return unpack(dataRequiredList)
end

function AloftNameText:SetupFrame(aloftData)
	local nameTextRegion = aloftData.nameTextRegion
	local healthBar = aloftData.healthBar
	local profile = self.db.profile

	nameTextRegion:ClearAllPoints()
	nameTextRegion:SetPoint("TOPLEFT", healthBar, "TOPLEFT", profile.offsets.left, profile.offsets.vertical)
	nameTextRegion:SetPoint("BOTTOMRIGHT", healthBar, "BOTTOMRIGHT", profile.offsets.right, profile.offsets.vertical)
	self:PrepareText(nameTextRegion, profile)

	if profile.enable then
		nameTextRegion:Show()
	else
		nameTextRegion:Hide()
	end
end

-----------------------------------------------------------------------------

function AloftNameText:SetTextColor(aloftData, color)
	aloftData.nameTextRegion:SetText(("|cff%02x%02x%02x%s"):format(color[1]*255, color[2]*255, color[3]*255, textMethod(aloftData)))
end

function AloftNameText:SetTextColorRgb(aloftData, color)
	aloftData.nameTextRegion:SetText(("|cff%02x%02x%02x%s"):format(color.r*255, color.g*255, color.b*255, textMethod(aloftData)))
end

-----------------------------------------------------------------------------

function AloftNameText:UpdateTextSimple(aloftData)
	if self.db.profile.colorByClass and aloftData.class then
		local color = RAID_CLASS_COLORS[aloftData.class]
		
		if color then
			self:SetTextColorRgb(aloftData, color)
			return
		end
	end
	
	self:SetTextColor(aloftData, self.db.profile.colors.friendlyPlayer)
end

-----------------------------------------------------------------------------

function AloftNameText:ColourFriendlyPlayer(aloftData)
	if self.db.profile.colorByClass then
		local class = aloftData.class
		if class then
			local color = RAID_CLASS_COLORS[class]
			if color then
				self:SetTextColorRgb(aloftData, color)
				return
			end
		end
	end
	self:SetTextColor(aloftData, self.db.profile.colors.friendlyPlayer)
end

function AloftNameText:ColourFriendlyNPC(aloftData)
	-- Could be player pet, group pet, friendly pet, friendlyBoss, or friendlyNPC
	if aloftData.name == UnitName("pet") then
		self:SetTextColor(aloftData, self.db.profile.colors.pet)
	elseif aloftData.unitid then
		self:SetTextColor(aloftData, self.db.profile.colors.groupPet)
	elseif aloftData.isPet then
		self:SetTextColor(aloftData, self.db.profile.colors.friendlyPet)
	elseif aloftData.isBoss then
		self:SetTextColor(aloftData, self.db.profile.colors.friendlyBoss)
	else
		self:SetTextColor(aloftData, self.db.profile.colors.friendlyNPC)
	end
end

function AloftNameText:ColourHostile(aloftData)
	-- player, hostile pet, hostile boss, hostileNPC
	if self.db.profile.colorHostileByClass then
		local class = aloftData.class
		if class then
			local color = RAID_CLASS_COLORS[class]
			if color then
				self:SetTextColorRgb(aloftData, color)
				return
			end
		end
	end

	if aloftData.class then
		self:SetTextColor(aloftData, self.db.profile.colors.hostilePlayer)
	elseif aloftData.isPet then
		self:SetTextColor(aloftData, self.db.profile.colors.hostilePet)
	elseif aloftData.isBoss then
		self:SetTextColor(aloftData, self.db.profile.colors.hostileBoss)
	else
		self:SetTextColor(aloftData, self.db.profile.colors.hostileNPC)
	end
end

function AloftNameText:ColourNeutral(aloftData)
	self:SetTextColor(aloftData, self.db.profile.colors.neutral)
end

function AloftNameText:ColourUnknown(aloftData)
end

local nameTextMethods =
{
	["friendlyPlayer"] = AloftNameText.ColourFriendlyPlayer,
	["friendlyNPC"] = AloftNameText.ColourFriendlyNPC,
	["hostile"] = AloftNameText.ColourHostile,
	["neutral"] = AloftNameText.ColourNeutral,
	["unknown"] = AloftNameText.ColourUnknown,
}

function AloftNameText:UpdateText(aloftData)
	nameTextMethods[aloftData.type](self, aloftData)
end

function AloftNameText:UpdateTextNone(aloftData)
	aloftData.nameTextRegion:SetText(textMethod(aloftData))
end

-----------------------------------------------------------------------------
